#pragma once

#include "ScalarAttributes.h"
#include "VectorAttributes.h"


class UnstructuredGrid3D;
class PointSearch;



class MeshPCA			//Compute the curvature tensor's eigen-components for the vertices of a grid.
{						//Next, compute three surface classification metrics based on this tensor.
public:
						MeshPCA(UnstructuredGrid3D&,int numNeighbors=100);
					   ~MeshPCA();	

void					compute(float radius);
						//Compute PCA on all neighborhoods of the grid's vertices of size 'radius'
						//Next, compute three surface classification metrics based on this tensor.

void					searchOnSurface(bool s)
						{ search_on_surface = s; }
						//Tell if the next compute() will search point-neighbors strictly on the mesh
						//surface or in 3D space. See search_on_surface.

ScalarAttributes		variation,planarity,classifier;			
											//Surface anisotropy metrics computed by this:
											//variation: Pauly's surface variation
											//planarity: Westin's surface planarity
											//classifier: Clarenz's surface momentum-based classifier

protected:

UnstructuredGrid3D&		grid;				//The grid we work on
PointSearch*			point_search;		//Spatial seach structure for grid vertices
int						numNeighbors;		//Maximal number of points in a neighborhood
bool					search_on_surface;	//Search points strictly on grid surface (slower, more accurate)
											//or in 3D space (faster, less accurate)
};

					
